@@ -16,9 +16,6 @@
  * Boston, MA 021110-1307, USA.
  */
 
-#define _XOPEN_SOURCE 600
-#define _GNU_SOURCE 1
-
 #include "kerncompat.h"
 
 #include <sys/ioctl.h>
@@ -32,6 +29,7 @@
 #include <unistd.h>
 #include <uuid/uuid.h>
 #include <linux/limits.h>
+#include <getopt.h>
 
 #include "ctree.h"
 #include "disk-io.h"
@@ -2762,14 +2760,15 @@ fail:
 
 static void print_usage(void)
 {
-	printf("usage: btrfs-convert [-d] [-i] [-n] [-r] [-l label] [-L] device\n");
-	printf("\t-d           disable data checksum\n");
-	printf("\t-i           ignore xattrs and ACLs\n");
-	printf("\t-n           disable packing of small files\n");
-	printf("\t-r           roll back to ext2fs\n");
-	printf("\t-l LABEL     set filesystem label\n");
-	printf("\t-L           use label from converted fs\n");
-	printf("\t-p           show converting progress\n");
+	printf("usage: btrfs-convert [options] device\n");
+	printf("\t-d             disable data checksum\n");
+	printf("\t-i             ignore xattrs and ACLs\n");
+	printf("\t-n             disable packing of small files\n");
+	printf("\t-r             roll back to ext2fs\n");
+	printf("\t-l LABEL       set filesystem label\n");
+	printf("\t-L             use label from converted fs\n");
+	printf("\t-p             show converting progress (default)\n");
+	printf("\t--no-progress  show only overview, not the detailed progress\n");
 }
 
 int main(int argc, char *argv[])
@@ -2781,12 +2780,20 @@ int main(int argc, char *argv[])
 	int rollback = 0;
 	int copylabel = 0;
 	int usage_error = 0;
-	int progress = 0;
+	int progress = 1;
 	char *file;
 	char *fslabel = NULL;
 
 	while(1) {
-		int c = getopt(argc, argv, "dinrl:Lp");
+		int long_index;
+		enum { GETOPT_VAL_NO_PROGRESS = 256 };
+		static const struct option long_options[] = {
+			{ "no-progress", no_argument, NULL, GETOPT_VAL_IEC},
+			{ NULL, 0, NULL, 0 }
+		};
+		int c = getopt_long(argc, argv, "dinrl:Lp", long_options,
+				&long_index);
+
 		if (c < 0)
 			break;
 		switch(c) {
@@ -2818,6 +2825,9 @@ int main(int argc, char *argv[])
 			case 'p':
 				progress = 1;
 				break;
+			case GETOPT_VAL_NO_PROGRESS:
+				progress = 0;
+				break;
 			default:
 				print_usage();
 				return 1;
