@@ -28,6 +28,7 @@
 #include "ctree.h"
 #include "ioctl.h"
 #include "utils.h"
+#include "cmds-fi-disk_usage.h"
 
 #include "commands.h"
 
@@ -55,10 +56,10 @@ static int cmd_add_dev(int argc, char **argv)
 
 	while (1) {
 		int long_index;
-		static struct option long_options[] = {
+		static const struct option long_options[] = {
 			{ "nodiscard", optional_argument, NULL, 'K'},
 			{ "force", no_argument, NULL, 'f'},
-			{ 0, 0, 0, 0 }
+			{ NULL, 0, NULL, 0}
 		};
 		int c = getopt_long(argc, argv, "Kf", long_options,
 					&long_index);
@@ -213,9 +214,9 @@ static int cmd_scan_dev(int argc, char **argv)
 	optind = 1;
 	while (1) {
 		int long_index;
-		static struct option long_options[] = {
+		static const struct option long_options[] = {
 			{ "all-devices", no_argument, NULL, 'd'},
-			{ 0, 0, 0, 0 },
+			{ NULL, 0, NULL, 0}
 		};
 		int c = getopt_long(argc, argv, "d", long_options,
 				    &long_index);
@@ -408,31 +409,37 @@ static int cmd_dev_stats(int argc, char **argv)
 				path, strerror(errno));
 			err = 1;
 		} else {
+			char *canonical_path;
+
+			canonical_path = canonicalize_path((char *)path);
+
 			if (args.nr_items >= BTRFS_DEV_STAT_WRITE_ERRS + 1)
 				printf("[%s].write_io_errs   %llu\n",
-				       path,
+				       canonical_path,
 				       (unsigned long long) args.values[
 					BTRFS_DEV_STAT_WRITE_ERRS]);
 			if (args.nr_items >= BTRFS_DEV_STAT_READ_ERRS + 1)
 				printf("[%s].read_io_errs    %llu\n",
-				       path,
+				       canonical_path,
 				       (unsigned long long) args.values[
 					BTRFS_DEV_STAT_READ_ERRS]);
 			if (args.nr_items >= BTRFS_DEV_STAT_FLUSH_ERRS + 1)
 				printf("[%s].flush_io_errs   %llu\n",
-				       path,
+				       canonical_path,
 				       (unsigned long long) args.values[
 					BTRFS_DEV_STAT_FLUSH_ERRS]);
 			if (args.nr_items >= BTRFS_DEV_STAT_CORRUPTION_ERRS + 1)
 				printf("[%s].corruption_errs %llu\n",
-				       path,
+				       canonical_path,
 				       (unsigned long long) args.values[
 					BTRFS_DEV_STAT_CORRUPTION_ERRS]);
 			if (args.nr_items >= BTRFS_DEV_STAT_GENERATION_ERRS + 1)
 				printf("[%s].generation_errs %llu\n",
-				       path,
+				       canonical_path,
 				       (unsigned long long) args.values[
 					BTRFS_DEV_STAT_GENERATION_ERRS]);
+
+			free(canonical_path);
 		}
 	}
 
@@ -443,6 +450,139 @@ out:
 	return err;
 }
 
+const char * const cmd_device_usage_usage[] = {
+	"btrfs device usage [options] <path> [<path>..]",
+	"Show detailed information about internal allocations in devices.",
+	"-b|--raw           raw numbers in bytes",
+	"-h|--human-readable",
+	"                   human friendly numbers, base 1024 (default)",
+	"-H                 human friendly numbers, base 1000",
+	"--iec              use 1024 as a base (KiB, MiB, GiB, TiB)",
+	"--si               use 1000 as a base (kB, MB, GB, TB)",
+	"-k|--kbytes        show sizes in KiB, or kB with --si",
+	"-m|--mbytes        show sizes in MiB, or MB with --si",
+	"-g|--gbytes        show sizes in GiB, or GB with --si",
+	"-t|--tbytes        show sizes in TiB, or TB with --si",
+	NULL
+};
+
+static int _cmd_device_usage(int fd, char *path, unsigned unit_mode)
+{
+	int i;
+	int ret = 0;
+	struct chunk_info *chunkinfo = NULL;
+	struct device_info *devinfo = NULL;
+	int chunkcount = 0;
+	int devcount = 0;
+
+	ret = load_chunk_and_device_info(fd, &chunkinfo, &chunkcount, &devinfo,
+			&devcount);
+	if (ret)
+		goto out;
+
+	for (i = 0; i < devcount; i++) {
+		printf("%s, ID: %llu\n", devinfo[i].path, devinfo[i].devid);
+		print_device_sizes(fd, &devinfo[i], unit_mode);
+		print_device_chunks(fd, &devinfo[i], chunkinfo, chunkcount,
+				unit_mode);
+		printf("\n");
+	}
+
+out:
+	free(devinfo);
+	free(chunkinfo);
+
+	return ret;
+}
+
+int cmd_device_usage(int argc, char **argv)
+{
+	unsigned unit_mode = UNITS_DEFAULT;
+	int ret = 0;
+	int	i, more_than_one = 0;
+
+	optind = 1;
+	while (1) {
+		int long_index;
+		static const struct option long_options[] = {
+			{ "raw", no_argument, NULL, 'b'},
+			{ "kbytes", no_argument, NULL, 'k'},
+			{ "mbytes", no_argument, NULL, 'm'},
+			{ "gbytes", no_argument, NULL, 'g'},
+			{ "tbytes", no_argument, NULL, 't'},
+			{ "si", no_argument, NULL, GETOPT_VAL_SI},
+			{ "iec", no_argument, NULL, GETOPT_VAL_IEC},
+			{ "human-readable", no_argument, NULL,
+				GETOPT_VAL_HUMAN_READABLE},
+			{ NULL, 0, NULL, 0 }
+		};
+		int c = getopt_long(argc, argv, "bhHkmgt", long_options,
+				&long_index);
+
+		if (c < 0)
+			break;
+		switch (c) {
+		case 'b':
+			unit_mode = UNITS_RAW;
+			break;
+		case 'k':
+			units_set_base(&unit_mode, UNITS_KBYTES);
+			break;
+		case 'm':
+			units_set_base(&unit_mode, UNITS_MBYTES);
+			break;
+		case 'g':
+			units_set_base(&unit_mode, UNITS_GBYTES);
+			break;
+		case 't':
+			units_set_base(&unit_mode, UNITS_TBYTES);
+			break;
+		case GETOPT_VAL_HUMAN_READABLE:
+		case 'h':
+			unit_mode = UNITS_HUMAN_BINARY;
+			break;
+		case 'H':
+			unit_mode = UNITS_HUMAN_DECIMAL;
+			break;
+		case GETOPT_VAL_SI:
+			units_set_mode(&unit_mode, UNITS_DECIMAL);
+			break;
+		case GETOPT_VAL_IEC:
+			units_set_mode(&unit_mode, UNITS_BINARY);
+			break;
+		default:
+			usage(cmd_device_usage_usage);
+		}
+	}
+
+	if (check_argc_min(argc - optind, 1))
+		usage(cmd_device_usage_usage);
+
+	for (i = optind; i < argc ; i++) {
+		int fd;
+		DIR	*dirstream = NULL;
+		if (more_than_one)
+			printf("\n");
+
+		fd = open_file_or_dir(argv[i], &dirstream);
+		if (fd < 0) {
+			fprintf(stderr, "ERROR: can't access '%s'\n",
+				argv[1]);
+			ret = 1;
+			goto out;
+		}
+
+		ret = _cmd_device_usage(fd, argv[i], unit_mode);
+		close_file_or_dir(fd, dirstream);
+
+		if (ret)
+			goto out;
+		more_than_one = 1;
+	}
+out:
+	return !!ret;
+}
+
 const struct cmd_group device_cmd_group = {
 	device_cmd_group_usage, NULL, {
 		{ "add", cmd_add_dev, cmd_add_dev_usage, NULL, 0 },
@@ -450,6 +590,8 @@ const struct cmd_group device_cmd_group = {
 		{ "scan", cmd_scan_dev, cmd_scan_dev_usage, NULL, 0 },
 		{ "ready", cmd_ready_dev, cmd_ready_dev_usage, NULL, 0 },
 		{ "stats", cmd_dev_stats, cmd_dev_stats_usage, NULL, 0 },
+		{ "usage", cmd_device_usage,
+			cmd_device_usage_usage, NULL, 0 },
 		NULL_CMD_STRUCT
 	}
 };
