@@ -16,9 +16,6 @@
  * Boston, MA 021110-1307, USA.
  */
 
-#define _XOPEN_SOURCE 500
-#define _GNU_SOURCE
-
 #include "kerncompat.h"
 
 #include <sys/ioctl.h>
@@ -44,7 +41,6 @@
 #include "volumes.h"
 #include "transaction.h"
 #include "utils.h"
-#include "version.h"
 
 static u64 index_cnt = 2;
 
@@ -293,14 +289,14 @@ static void print_usage(void)
 	fprintf(stderr, "\t -O --features comma separated list of filesystem features\n");
 	fprintf(stderr, "\t -U --uuid specify the filesystem UUID\n");
 	fprintf(stderr, "\t -V --version print the mkfs.btrfs version and exit\n");
-	fprintf(stderr, "%s\n", BTRFS_BUILD_VERSION);
+	fprintf(stderr, "%s\n", PACKAGE_STRING);
 	exit(1);
 }
 
 static void print_version(void) __attribute__((noreturn));
 static void print_version(void)
 {
-	fprintf(stderr, "mkfs.btrfs, part of %s\n", BTRFS_BUILD_VERSION);
+	fprintf(stderr, "mkfs.btrfs, part of %s\n", PACKAGE_STRING);
 	exit(0);
 }
 
@@ -340,25 +336,6 @@ static char *parse_label(char *input)
 	return strdup(input);
 }
 
-static struct option long_options[] = {
-	{ "alloc-start", 1, NULL, 'A'},
-	{ "byte-count", 1, NULL, 'b' },
-	{ "force", 0, NULL, 'f' },
-	{ "leafsize", 1, NULL, 'l' },
-	{ "label", 1, NULL, 'L'},
-	{ "metadata", 1, NULL, 'm' },
-	{ "mixed", 0, NULL, 'M' },
-	{ "nodesize", 1, NULL, 'n' },
-	{ "sectorsize", 1, NULL, 's' },
-	{ "data", 1, NULL, 'd' },
-	{ "version", 0, NULL, 'V' },
-	{ "rootdir", 1, NULL, 'r' },
-	{ "nodiscard", 0, NULL, 'K' },
-	{ "features", 1, NULL, 'O' },
-	{ "uuid", required_argument, NULL, 'U' },
-	{ NULL, 0, NULL, 0}
-};
-
 static int add_directory_items(struct btrfs_trans_handle *trans,
 			       struct btrfs_root *root, u64 objectid,
 			       ino_t parent_inum, const char *name,
@@ -1258,7 +1235,6 @@ int main(int ac, char **av)
 	u32 nodesize = leafsize;
 	u32 stripesize = 4096;
 	int zero_end = 1;
-	int option_index = 0;
 	int fd;
 	int ret;
 	int i;
@@ -1283,6 +1259,26 @@ int main(int ac, char **av)
 
 	while(1) {
 		int c;
+		int option_index = 0;
+		static const struct option long_options[] = {
+			{ "alloc-start", 1, NULL, 'A'},
+			{ "byte-count", 1, NULL, 'b' },
+			{ "force", 0, NULL, 'f' },
+			{ "leafsize", 1, NULL, 'l' },
+			{ "label", 1, NULL, 'L'},
+			{ "metadata", 1, NULL, 'm' },
+			{ "mixed", 0, NULL, 'M' },
+			{ "nodesize", 1, NULL, 'n' },
+			{ "sectorsize", 1, NULL, 's' },
+			{ "data", 1, NULL, 'd' },
+			{ "version", 0, NULL, 'V' },
+			{ "rootdir", 1, NULL, 'r' },
+			{ "nodiscard", 0, NULL, 'K' },
+			{ "features", 1, NULL, 'O' },
+			{ "uuid", required_argument, NULL, 'U' },
+			{ NULL, 0, NULL, 0}
+		};
+
 		c = getopt_long(ac, av, "A:b:fl:n:s:m:d:L:O:r:U:VMK",
 				long_options, &option_index);
 		if (c < 0)
@@ -1492,8 +1488,8 @@ int main(int ac, char **av)
 	}
 
 	/* if we are here that means all devs are good to btrfsify */
-	printf("%s\n", BTRFS_BUILD_VERSION);
-	printf("See http://btrfs.wiki.kernel.org for more information.\n\n");
+	printf("%s\n", PACKAGE_STRING);
+	printf("See %s for more information.\n\n", PACKAGE_URL);
 
 	dev_cnt--;
 
